@@ -1553,14 +1553,21 @@ class InvokedTask(DockerMixin, Model):
             self.cache['task_definition'] = TaskDefinition.objects.get(self.data['taskDefinitionArn'])
         return self.cache['task_definition']
 
+    @property
+    def instance(self):
+        return self.container_instance.ec2_instance
+
     @property
     def ssh_target(self):
         """
         .. warning::
 
-            If this is a FARGATE task, we may have a container instance, but we won't be able to ssh to it.
+            If this is a FARGATE task, we won't have a container instance.
         """
-        return self.container_instance.ec2_instance
+        if self.container_instance:
+            return self.container_instance.ec2_instance
+        else:
+            return None
 
     @property
     def cluster_name(self):
@@ -1573,16 +1580,26 @@ class InvokedTask(DockerMixin, Model):
     @property
     def container_instance(self):
         # This may not work for FARGATE tasks -- do they return containerInstanceArn?
-        return self.get_cached(
-            'container_machine',
-            ContainerInstance.objects.get,
-            ['{}:{}'.format(self.cluster_name, self.data['containerInstanceArn'])]
-        )
+        try:
+            return self.get_cached(
+                'container_machine',
+                ContainerInstance.objects.get,
+                ['{}:{}'.format(self.cluster_name, self.data['containerInstanceArn'])]
+            )
+        except KeyError:
+            # this is a FARGATE task
+            return None
 
     def render_for_display(self):
         data = self.render()
         data['version'] = self.task_definition.version
         data['cluster'] = self.cluster_name
+        if self.container_instance:
+            data['instanceName'] = self.container_instance.name
+            data['instanceId'] = self.container_instance.ec2_instance.pk
+        else:
+            data['instanceName'] = ''
+            data['instanceId'] = ''
         data['taskDefinition'] = self.task_definition.render_for_display()
         return data
 
@@ -1692,8 +1709,8 @@ class Cluster(TagsMixin, SSHMixin, Model):
         return responses
 
     @property
-    def tasks(self):
-        return InvokedTask.objects.list
+    def running_tasks(self):
+        return InvokedTask.objects.list(self.name)
 
     @property
     def services(self):
@@ -1704,13 +1721,13 @@ class Cluster(TagsMixin, SSHMixin, Model):
         if 'autoscaling_group' not in self.cache:
             if len(self.container_instances) > 0:
                 self.cache['autoscaling_group'] = self.container_instances[0].autoscaling_group
-        else:
-            # Try to guess our autoscaling group based on our cluster name
-            try:
-                self.cache['autoscaling_group'] = AutoscalingGroup.objects.get(self.name)
-            except AutoscalingGroup.DoesNotExist:
-                self.cache['autoscaling_group'] = None
-        return self.cache['autoscaling_group']
+            else:
+                # Try to guess our autoscaling group based on our cluster name
+                try:
+                    self.cache['autoscaling_group'] = AutoscalingGroup.objects.get(self.name)
+                except AutoscalingGroup.DoesNotExist:
+                    self.cache['autoscaling_group'] = None
+        return self.cache.get('autoscaling_group', None)
 
     def scale(self, count, force=True):
         if self.autoscaling_group:
@@ -1937,12 +1954,7 @@ class Service(TagsMixin, DockerMixin, SecretsMixin, Model):
 
     @property
     def running_tasks(self):
-        return self.get_cached(
-            'running_tasks',
-            InvokedTask.objects.list,
-            [self.data['cluster']],
-            {'service': self.data['serviceName']}
-        )
+        return InvokedTask.objects.list(self.data['cluster'], service=self.name)
 
     @property
     def container_instances(self):
